from dune.perftool.generation.cache import _RegisteredFunction
from dune.perftool.options import option_switch
from pytools import Record


_backend_mapping = {}


class FuncProxy(Record):
    def __init__(self, interface, name, func):
        Record.__init__(self, interface=interface, name=name, func=func)

    def __call__(self, *args, **kwargs):
        return self.func(*args, **kwargs)


def register_backend(interface, name, func):
    _backend_mapping.setdefault(interface, {})
    _backend_mapping[interface][name] = func


def backend(interface=None, name='default'):
    assert interface

    def _dec(func):
        if not isinstance(func, _RegisteredFunction):
            # Allow order independence of the generator decorators
            # and the backend decorator by delaying the registration
            func = FuncProxy(interface, name, func)

        register_backend(interface, name, func)

        return func

    return _dec


def get_backend(interface=None, selector=option_switch("sumfact"), **kwargs):
    assert interface and selector

    select = selector(**kwargs)
    assert select in _backend_mapping[interface], "Implementation '{}' for interface '{}' missing!".format(select, interface)

    return _backend_mapping[interface][select]
