""" Context managers for code generation. """

_global_context_cache = {}


class _GlobalContext(object):
    def __init__(self, **kwargs):
        self.kw = kwargs

    def __enter__(self):
        self.old_kw = {}
        for k, v in self.kw.items():
            # First store existing values of the same keys
            if k in _global_context_cache:
                self.old_kw[k] = _global_context_cache[k]
            # Now replace the value with the new one
            _global_context_cache[k] = v

    def __exit__(self, exc_type, exc_value, traceback):
        # Delete all the entries from this context
        for k in self.kw.keys():
            del _global_context_cache[k]
        # and restore previously overwritten values
        for k, v in self.old_kw.items():
            _global_context_cache[k] = v


def global_context(**kwargs):
    return _GlobalContext(**kwargs)


def get_global_context_value(key, default=None):
    return _global_context_cache.get(key, default)
