"""
Class for temporary variables that allows us to use very non-standard types.
"""
from dune.perftool.error import PerftoolLoopyError

from loopy import TemporaryVariable

import numpy


def _temporary_type(shape_impl, shape, first=True):
    from dune.perftool.loopy.target import type_floatingpoint
    if len(shape_impl) == 0:
        return type_floatingpoint()
    if shape_impl[0] == 'arr':
        if not first or len(set(shape_impl)) != 1:
            raise PerftoolLoopyError("We do not allow mixing of C++ containers and plain C arrays, for reasons of mental sanity")
        return type_floatingpoint()
    if shape_impl[0] == 'vec':
        return "std::vector<{}>".format(_temporary_type(shape_impl[1:], shape[1:], first=False))
    if shape_impl[0] == 'fv':
        return "Dune::FieldVector<{}, {}>".format(_temporary_type(shape_impl[1:], shape[1:], first=False), shape[0])
    if shape_impl[0] == 'fm':
        # For now, no field matrices of weird stuff...
        assert len(shape) == 2
        _type = type_floatingpoint()
        return "Dune::FieldMatrix<{}, {}, {}>".format(_type, shape[0], shape[1])


def default_declaration(name, shape=(), shape_impl=()):
    # Determine the C++ type to use for this temporary.
    t = _temporary_type(shape_impl, shape)
    if len(shape_impl) == 0:
        # This is a scalar, just return it!
        return '{} {}(0.0);'.format(t, name)
    if shape_impl[0] == 'arr':
        return '{} {}{};'.format(t, name, ''.join('[{}]'.format(s) for s in shape))
    if shape_impl[0] == 'vec':
        return '{} {}({});'.format(t, name, shape[0])
    if shape_impl[0] == 'fv':
        return '{} {}(0.0);'.format(t, name)
    if shape_impl[0] == 'fm':
        assert len(shape) == 2
        return '{} {}(0.0);'.format(t, name)


class DuneTemporaryVariable(TemporaryVariable):

    allowed_extra_kwargs = TemporaryVariable.allowed_extra_kwargs + ["managed", "shape_impl", "decl_method"]

    def __init__(self, name, managed=False, shape_impl=None, decl_method=None, **kwargs):
        self.managed = managed
        self.decl_method = decl_method
        self.shape_impl = shape_impl

        if shape_impl is not None:
            self.decl_method = default_declaration

        from dune.perftool.loopy.target import dtype_floatingpoint
        kwargs.setdefault('dtype', dtype_floatingpoint())

        self.custom_declaration = self.decl_method is not None

        TemporaryVariable.__init__(self, name, managed=self.managed, shape_impl=self.shape_impl, decl_method=self.decl_method, **kwargs)
