from dune.perftool.generation import global_context, preamble
from dune.perftool.ufl.visitor import UFL2LoopyVisitor

import pymbolic.primitives as prim


@preamble
def driver_using_statement(what):
    return "using {};".format(what)


class DriverUFL2PymbolicVisitor(UFL2LoopyVisitor):
    def __init__(self):
        from dune.perftool.pdelab import PDELabInterface
        UFL2LoopyVisitor.__init__(self, PDELabInterface(), "exterior_facet", {})

    def __call__(self, expr):
        return self._call(expr, False)

    def spatial_coordinate(self, o):
        return prim.Variable("x")

    def max_value(self, o):
        driver_using_statement("std::max")
        return UFL2LoopyVisitor.max_value(self, o)

    def min_value(self, o):
        driver_using_statement("std::min")
        return UFL2LoopyVisitor.min_value(self, o)

    def coefficient(self, o):
        if o.count() == 2:
            from dune.perftool.pdelab.driver import get_form_ident
            from dune.perftool.pdelab.driver.gridoperator import name_localoperator
            lop = name_localoperator(get_form_ident())
            return prim.Call(prim.Variable("{}.getTime".format(lop)), ())
        else:
            return UFL2LoopyVisitor.coefficient(self, o)


def ufl_to_code(expr, boundary=True):
    # So far, we only considered this code branch on boundaries!
    assert boundary
    from dune.perftool.pdelab.driver import get_form_ident
    with global_context(integral_type="exterior_facet", form_identifier=get_form_ident()):
        visitor = DriverUFL2PymbolicVisitor()
        from pymbolic.mapper.c_code import CCodeMapper
        ccm = CCodeMapper()
        vis = visitor(expr)
        return ccm(vis)
