""" Code generation for explicitly specified tensors """

from dune.perftool.generation import (get_counted_variable,
                                      domain,
                                      kernel_cached,
                                      iname,
                                      instruction,
                                      temporary_variable,
                                      )

import pymbolic.primitives as prim
import numpy as np
import loopy as lp


def define_list_tensor(name, expr, visitor, stack=()):
    for i, child in enumerate(expr.ufl_operands):
        from ufl.classes import ListTensor
        if isinstance(child, ListTensor):
            define_list_tensor(name, child, visitor, stack=stack + (i,))
        else:
            visexpr = visitor.call(child)
            from loopy.symbolic import DependencyMapper
            deps = DependencyMapper(include_subscripts=False, include_lookups=False, include_calls=False)(visexpr)
            instruction(assignee=prim.Subscript(prim.Variable(name), stack + (i,)),
                        expression=visitor.call(child),
                        forced_iname_deps=frozenset(visitor.interface.quadrature_inames()),
                        depends_on=frozenset({lp.match.Tagged("sumfact_stage1")}),
                        tags=frozenset({"quad"}),
                        )


@kernel_cached
def pymbolic_list_tensor(expr, visitor):
    name = get_counted_variable("listtensor")
    temporary_variable(name,
                       shape=expr.ufl_shape,
                       managed=True,
                       )
    define_list_tensor(name, expr, visitor)
    return prim.Variable(name)


@iname
def identity_iname(name, bound):
    name = "id_{}_{}".format(name, bound)
    domain(name, bound)
    return name


def define_identity(name, expr):
    i = identity_iname("i", expr.ufl_shape[0])
    j = identity_iname("j", expr.ufl_shape[1])
    instruction(assignee=prim.Subscript(prim.Variable(name), (prim.Variable(i), prim.Variable(j))),
                expression=prim.If(prim.Comparison(prim.Variable(i), "==", prim.Variable(j)), 1, 0),
                forced_iname_deps_is_final=True,
                )


@kernel_cached
def pymbolic_identity(expr):
    name = "identity_{}_{}".format(expr.ufl_shape[0], expr.ufl_shape[1])
    temporary_variable(name,
                       shape=expr.ufl_shape,
                       shape_impl=('fm',),
                       )
    define_identity(name, expr)
    return prim.Variable(name)
