from dune.perftool.generation import get_backend
from dune.perftool.options import option_switch
from dune.perftool.pdelab.argument import (name_applycontainer,
                                           name_coefficientcontainer,
                                           )
from dune.perftool.sumfact.quadrature import (quadrature_inames,
                                              quadrature_weight,
                                              pymbolic_quadrature_position,
                                              )

from dune.perftool.sumfact.basis import (lfs_inames,
                                         pymbolic_basis,
                                         pymbolic_reference_gradient,
                                         pymbolic_coefficient,
                                         pymbolic_coefficient_gradient,
                                         )

import dune.perftool.sumfact.accumulation
import dune.perftool.sumfact.switch

from dune.perftool.pdelab import PDELabInterface


class SumFactInterface(PDELabInterface):
    def get_accumulation_info(self, expr, visitor):
        from dune.perftool.sumfact.accumulation import get_accumulation_info
        return get_accumulation_info(expr, visitor)

    def list_accumulation_infos(self, expr, visitor):
        from dune.perftool.sumfact.accumulation import list_accumulation_infos
        return list_accumulation_infos(expr, visitor)

    def generate_accumulation_instruction(self, expr, visitor):
        from dune.perftool.sumfact.accumulation import generate_accumulation_instruction
        return generate_accumulation_instruction(expr, visitor)

    def lfs_inames(self, element, restriction, number=None, context=''):
        return lfs_inames(element, restriction, number, context)

    def pymbolic_basis(self, element, restriction, number):
        return pymbolic_basis(element, restriction, number)

    def pymbolic_reference_gradient(self, element, restriction, number):
        ret, indices = pymbolic_reference_gradient(element, restriction, number, self.visitor.indices)
        self.visitor.indices = indices
        return ret

    def pymbolic_trialfunction_gradient(self, element, restriction, index):
        ret = pymbolic_coefficient_gradient(element, restriction, index, name_coefficientcontainer, self.visitor)
        return ret

    def pymbolic_trialfunction(self, element, restriction, index):
        ret = pymbolic_coefficient(element, restriction, index, name_coefficientcontainer, self.visitor)
        return ret

    def pymbolic_apply_function_gradient(self, element, restriction, index):
        ret = pymbolic_coefficient_gradient(element, restriction, index, name_applycontainer, self.visitor)
        return ret

    def pymbolic_apply_function(self, element, restriction, index):
        ret = pymbolic_coefficient(element, restriction, index, name_applycontainer, self.visitor)
        return ret

    def quadrature_inames(self):
        return quadrature_inames()

    def pymbolic_quadrature_weight(self):
        return quadrature_weight(self.visitor)

    #
    # Geometry related generator functions
    #

    def pymbolic_spatial_coordinate(self):
        import dune.perftool.sumfact.geometry
        ret = get_backend(interface="spatial_coordinate", selector=option_switch("diagonal_transformation_matrix"))(self.visitor.do_predicates, self.visitor)
        return ret

    def pymbolic_unit_outer_normal(self):
        from dune.perftool.sumfact.geometry import pymbolic_unit_outer_normal
        ret, indices = pymbolic_unit_outer_normal(self.visitor.indices)
        self.visitor.indices = indices
        return ret

    def pymbolic_unit_inner_normal(self):
        from dune.perftool.sumfact.geometry import pymbolic_unit_inner_normal
        ret, indices = pymbolic_unit_inner_normal(self.visitor.indices)
        self.visitor.indices = indices
        return ret

    def pymbolic_facet_jacobian_determinant(self):
        from dune.perftool.sumfact.geometry import pymbolic_facet_jacobian_determinant
        return pymbolic_facet_jacobian_determinant()

    def pymbolic_facet_area(self):
        from dune.perftool.sumfact.geometry import pymbolic_facet_area
        return pymbolic_facet_area()
